@@ -192,6 +192,8 @@ goc_item_dispose (GObject *object)
 	if (item->canvas) {
 		if (item->canvas->last_item == item)
 			item->canvas->last_item = NULL;
+		if (item->canvas->grabbed_item == item)
+			item->canvas->grabbed_item = NULL;
 		if (gtk_widget_is_realized (item->canvas)) {
 			item->cached_bounds = TRUE; /* avoids a call to update_bounds */
 			goc_item_invalidate (item);
