@@ -22,6 +22,7 @@
 #include <goffice/goffice-config.h>
 #include <goffice/goffice.h>
 #include <goffice/utils/go-marshalers.h>
+#include <goffice/gtk/go-gtk-compat.h>
 
 #include <gsf/gsf-impl-utils.h>
 #include <glib/gi18n-lib.h>
@@ -36,20 +37,20 @@
 #define START_LOCALE_SWITCH							\
 	do {									\
 		if (gfs->locale) {						\
-			go_currency_date_format_shutdown ();			\
+			_go_currency_date_format_shutdown ();			\
 			oldlocale = g_strdup (setlocale (LC_ALL, NULL));	\
 			go_setlocale (LC_ALL, gfs->locale);			\
-			go_currency_date_format_init ();			\
+			_go_currency_date_format_init ();			\
 		}								\
 	} while (0)
 
 #define END_LOCALE_SWITCH							\
 	do {									\
 		if (oldlocale) {						\
-			go_currency_date_format_shutdown ();			\
+			_go_currency_date_format_shutdown ();			\
 			go_setlocale (LC_ALL, oldlocale);			\
 			g_free (oldlocale);					\
-			go_currency_date_format_init ();			\
+			_go_currency_date_format_init ();			\
 		}								\
 	} while (0)
 
@@ -91,7 +92,7 @@ typedef enum {
 	F_SYMBOL_LABEL,		F_SYMBOL,
 	F_ENTRY,
 	F_LIST_LABEL,		F_LIST_SCROLL,		F_LIST,
-	F_DECIMAL_SPIN,		F_ENGINEERING_BUTTON,	
+	F_DECIMAL_SPIN,		F_ENGINEERING_BUTTON,
 	F_SUPERSCRIPT_BUTTON,	F_SUPERSCRIPT_HIDE_1_BUTTON,
 	F_NEGATIVE_LABEL,	F_NEGATIVE_SCROLL,	F_NEGATIVE,
 	F_DECIMAL_LABEL,	F_CODE_LABEL,
@@ -100,7 +101,7 @@ typedef enum {
 
 struct  _GOFormatSel {
 	GtkHBox   box;
-	GladeXML *gui;
+	GtkBuilder *gui;
 
 	gpointer  value;
 	char	 *locale;
@@ -209,7 +210,7 @@ draw_format_preview (GOFormatSel *gfs, gboolean regen_format)
 	else
 		gdk_color_parse ("black", &gdk_color);
 
-	gtk_widget_modify_text (GTK_WIDGET (gfs->format.preview), 
+	gtk_widget_modify_text (GTK_WIDGET (gfs->format.preview),
 				GTK_STATE_NORMAL, &gdk_color);
 	g_free (preview);
 }
@@ -283,7 +284,7 @@ cb_decimals_changed (GtkSpinButton *spin, GOFormatSel *gfs)
 	gfs->format.details.num_decimals =
 		gtk_spin_button_get_value_as_int (spin);
 
-	if (GTK_WIDGET_VISIBLE (gfs->format.widget[F_NEGATIVE]))
+	if (gtk_widget_get_visible (gfs->format.widget[F_NEGATIVE]))
 		fillin_negative_samples (gfs);
 
 	draw_format_preview (gfs, TRUE);
@@ -295,7 +296,7 @@ cb_separator_toggle (GtkObject *obj, GOFormatSel *gfs)
 	gfs->format.details.thousands_sep =
 		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (obj));
 
-	if (GTK_WIDGET_VISIBLE (gfs->format.widget[F_NEGATIVE]))
+	if (gtk_widget_get_visible (gfs->format.widget[F_NEGATIVE]))
 		fillin_negative_samples (gfs);
 
 	draw_format_preview (gfs, TRUE);
@@ -371,7 +372,7 @@ find_builtin (const char *fmtstr, int page, gboolean def)
 	if (page == GO_FORMAT_UNKNOWN)
 		return NULL;
 
-	candidates = go_format_builtins[page];
+	candidates = _go_format_builtins[page];
 	if (!candidates)
 		return NULL;
 
@@ -560,7 +561,7 @@ stays:
 			gtk_list_store_clear (gfs->format.formats.model);
 			for (; start <= end ; ++start)
 				fmt_dialog_init_fmt_list (gfs,
-					go_format_builtins[start], &select);
+					_go_format_builtins[start], &select);
 
 			/* If this is the custom page and the format has
 			* not been found append it */
@@ -611,13 +612,13 @@ stays:
 			if (gfs->show_format_with_markup) {
 				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
 							      gfs->format.details.use_markup);
-			} else 
+			} else
 				show_widget = FALSE;
 			break;
 
 		case F_SUPERSCRIPT_HIDE_1_BUTTON:
 			if (gfs->show_format_with_markup) {
-				
+
 				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
 							      gfs->format.details.simplify_mantissa);
 				gtk_widget_set_sensitive (w, gfs->format.details.use_markup);
@@ -641,9 +642,9 @@ stays:
 		}
 
 		if (show_widget) {
-			while (!GTK_WIDGET_VISIBLE (w)) {
+			while (!gtk_widget_get_visible (w)) {
 				gtk_widget_show (w);
-				w = w->parent;
+				w = gtk_widget_get_parent (w);
 			}
 		}
 	}
@@ -659,15 +660,15 @@ stays:
  */
 
 static void
-cb_format_class_changed (G_GNUC_UNUSED GtkTreeSelection *ignored, 
+cb_format_class_changed (G_GNUC_UNUSED GtkTreeSelection *ignored,
 			 GOFormatSel *gfs)
 {
 	int selected_item = 0;
 	GList *list;
-	GtkTreeSelection *selection = gtk_tree_view_get_selection 
+	GtkTreeSelection *selection = gtk_tree_view_get_selection
 		(GTK_TREE_VIEW(gfs->format.menu));
 
-	list = gtk_tree_selection_get_selected_rows 
+	list = gtk_tree_selection_get_selected_rows
 		(selection, &gfs->format.menu_model);
 	if (list) {
 		GtkTreePath *path;
@@ -750,15 +751,15 @@ cb_format_currency_select (G_GNUC_UNUSED GtkWidget *ct,
 	if (!gfs->enable_edit || new_text == NULL || *new_text == '\0')
 		return FALSE;
 
-	for (i = 0; go_format_currencies[i].symbol != NULL ; ++i) {
-		GOFormatCurrency const *ci = go_format_currencies + i;
+	for (i = 0; _go_format_currencies[i].symbol != NULL ; ++i) {
+		GOFormatCurrency const *ci = _go_format_currencies + i;
 		if (!strcmp (_(ci->description), new_text)) {
 			gfs->format.details.currency = ci;
 			break;
 		}
 	}
 
-	if (GTK_WIDGET_VISIBLE (gfs->format.widget[F_NEGATIVE]))
+	if (gtk_widget_get_visible (gfs->format.widget[F_NEGATIVE]))
 		fillin_negative_samples (gfs);
 
 	draw_format_preview (gfs, TRUE);
@@ -816,7 +817,7 @@ static void
 set_format_category (GOFormatSel *gfs, int row)
 {
 	GtkTreePath *path;
-	GtkTreeSelection *selection = gtk_tree_view_get_selection 
+	GtkTreeSelection *selection = gtk_tree_view_get_selection
 		((GTK_TREE_VIEW(gfs->format.menu)));
 
 	path = gtk_tree_path_new_from_indices (row, -1);
@@ -878,16 +879,16 @@ populate_menu (GOFormatSel *gfs)
 	GtkCellRenderer *renderer;
 	char const * const *categories = format_category_names;
 
-	gfs->format.menu_model = GTK_TREE_MODEL (gtk_list_store_new 
+	gfs->format.menu_model = GTK_TREE_MODEL (gtk_list_store_new
 						 (1, G_TYPE_STRING));
-	gtk_tree_view_set_model (GTK_TREE_VIEW (gfs->format.menu), 
+	gtk_tree_view_set_model (GTK_TREE_VIEW (gfs->format.menu),
 				 gfs->format.menu_model);
-	selection = gtk_tree_view_get_selection 
+	selection = gtk_tree_view_get_selection
 		(GTK_TREE_VIEW(gfs->format.menu));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_BROWSE);
 
 	while (*categories) {
-		gtk_list_store_append 
+		gtk_list_store_append
 			(GTK_LIST_STORE (gfs->format.menu_model), &iter);
 		gtk_list_store_set (GTK_LIST_STORE (gfs->format.menu_model),
 				    &iter, 0, _(*categories), -1);
@@ -954,21 +955,17 @@ nfs_init (GOFormatSel *gfs)
 	GOFormatFamily page;
 
 	GtkWidget *toplevel;
-	GtkWidget *old_parent;
 
 	gfs->enable_edit = FALSE;
 	gfs->show_format_with_markup = FALSE;
 	gfs->locale = NULL;
 
-	gfs->gui = go_glade_new ("go-format-sel.glade", NULL, GETTEXT_PACKAGE, NULL);
+	gfs->gui = go_gtk_builder_new ("go-format-sel.ui", GETTEXT_PACKAGE, NULL);
 	if (gfs->gui == NULL)
 		return;
 
-	toplevel = glade_xml_get_widget (gfs->gui, "number_box");
-	old_parent = gtk_widget_get_toplevel (toplevel);
-	gtk_widget_reparent (toplevel, GTK_WIDGET (gfs));
-	gtk_widget_destroy (old_parent);
-	gtk_widget_queue_resize (toplevel);
+	toplevel = go_gtk_builder_get_widget (gfs->gui, "number_box");
+	gtk_box_pack_start (GTK_BOX (gfs), toplevel, TRUE, TRUE, 0);
 
 	gfs->format.spec = go_format_general ();
 	go_format_ref (gfs->format.spec);
@@ -981,8 +978,8 @@ nfs_init (GOFormatSel *gfs)
 
 	study_format (gfs);
 
-	gfs->format.preview_box = glade_xml_get_widget (gfs->gui, "preview_box");
-	gfs->format.preview = GTK_TEXT_VIEW (glade_xml_get_widget (gfs->gui, "preview"));
+	gfs->format.preview_box = go_gtk_builder_get_widget (gfs->gui, "preview_box");
+	gfs->format.preview = GTK_TEXT_VIEW (gtk_builder_get_object (gfs->gui, "preview"));
 	{
 		PangoFontMetrics *metrics;
 		PangoContext *context;
@@ -1000,12 +997,14 @@ nfs_init (GOFormatSel *gfs)
 	}
 	gfs->format.preview_buffer = gtk_text_view_get_buffer (gfs->format.preview);
 
-	gfs->format.menu = glade_xml_get_widget (gfs->gui, "format_menu");
+	gfs->format.menu = go_gtk_builder_get_widget (gfs->gui, "format_menu");
 	populate_menu (gfs);
 
 	/* Collect all the required format widgets and hide them */
 	for (i = 0; (name = widget_names[i]) != NULL; ++i) {
-		tmp = glade_xml_get_widget (gfs->gui, name);
+		if (i == F_SYMBOL)
+			continue;
+		tmp = go_gtk_builder_get_widget (gfs->gui, name);
 
 		if (tmp == NULL) {
 			g_warning ("nfs_init : failed to load widget %s", name);
@@ -1017,6 +1016,9 @@ nfs_init (GOFormatSel *gfs)
 		gfs->format.widget[i] = tmp;
 	}
 
+	gfs->format.widget[F_SYMBOL] = go_combo_text_new (NULL);
+	gtk_widget_hide (gfs->format.widget[F_SYMBOL]);
+
 	/* set minimum heights */
 	gtk_widget_set_size_request (gfs->format.widget[F_LIST], -1, 100);
 	gtk_widget_set_size_request (gfs->format.widget[F_NEGATIVE], -1, 100);
@@ -1088,8 +1090,8 @@ nfs_init (GOFormatSel *gfs)
 		GSList *ptr, *l = NULL;
 		const char *desc;
 
-		for (i = 0; go_format_currencies[i].symbol != NULL ; ++i)
-			l = g_slist_prepend (l, _((gchar *)go_format_currencies[i].description));
+		for (i = 0; _go_format_currencies[i].symbol != NULL ; ++i)
+			l = g_slist_prepend (l, _((gchar *)_go_format_currencies[i].description));
 		l = g_slist_sort (l, funny_currency_order);
 
 		for (ptr = l; ptr != NULL ; ptr = ptr->next)
@@ -1106,8 +1108,11 @@ nfs_init (GOFormatSel *gfs)
 		g_signal_connect (G_OBJECT (combo), "entry_changed",
 			G_CALLBACK (cb_format_currency_select), gfs);
 		gtk_label_set_mnemonic_widget (
-			GTK_LABEL (glade_xml_get_widget (gfs->gui, "format_symbol_label")),
+			GTK_LABEL (gtk_builder_get_object (gfs->gui, "format_symbol_label")),
 			GTK_WIDGET (combo));
+		/* add the combo to its container */
+		gtk_box_pack_start (GTK_BOX (gtk_builder_get_object (gfs->gui, "format_symbol_box")),
+				    GTK_WIDGET (combo), TRUE, TRUE, 0);
 	}
 
 	/* Setup special handler for Custom */
@@ -1134,10 +1139,8 @@ go_format_sel_finalize (GObject *obj)
 	g_free (gfs->locale);
 	gfs->locale = NULL;
 
-	if (gfs->format.spec) {
-		go_format_unref (gfs->format.spec);
-		gfs->format.spec = NULL;
-	}
+	go_format_unref (gfs->format.spec);
+	gfs->format.spec = NULL;
 
 	if (gfs->format.size_group) {
 		g_object_unref (gfs->format.size_group);
@@ -1193,15 +1196,15 @@ nfs_class_init (GObjectClass *klass)
 GSF_CLASS (GOFormatSel, go_format_sel,
 	   nfs_class_init, nfs_init, GTK_TYPE_HBOX)
 
-/** 
+/**
  * go_format_sel_new_full:
  * @use_markup: enable formats using pango markup
  *
  * Creates a format selector widget, with general format selected
  * by default. When @use_markup is set to %TRUE, it shows additional
- * widgets for editing properties of formats using pango markup 
+ * widgets for editing properties of formats using pango markup
  * (e.g. scientific format with superscripted exponent).
- * 
+ *
  * returns: a format selector widget.
  **/
 
@@ -1224,7 +1227,7 @@ go_format_sel_new_full (gboolean use_markup)
  * Creates a format selector widget, with general format selected
  * by default, and formats using pango markup disabled. See
  * @go_format_sel_new_full.
- * 
+ *
  * returns: a format selector widget.
  **/
 
@@ -1325,8 +1328,8 @@ go_format_sel_editable_enters (GOFormatSel *gfs,
 	go_gtk_editable_enters (window, gfs->format.widget[F_ENTRY]);
 }
 
-void		
-go_format_sel_set_locale (GOFormatSel *gfs, 
+void
+go_format_sel_set_locale (GOFormatSel *gfs,
 			  char const *locale)
 {
 	g_free (gfs->locale);
@@ -1337,7 +1340,7 @@ go_format_sel_set_locale (GOFormatSel *gfs,
 
 /* The following utility function should possibly be in format.h but we */
 /* access to the array of category names which are better located here. */
-char const *    
+char const *
 go_format_sel_format_classification (GOFormat const *style_format)
 {
 	GOFormatFamily page = go_format_get_family (style_format);
