@@ -22,8 +22,7 @@
  */
 
 #include <goffice/goffice-config.h>
-#include "go-file.h"
-#include "go-glib-extras.h"
+#include <goffice/goffice.h>
 #include <gsf/gsf-input-memory.h>
 #include <gsf/gsf-input-stdio.h>
 #include <gsf/gsf-output-stdio.h>
@@ -52,7 +51,7 @@
 #include <time.h>
 
 #ifdef G_OS_WIN32
-typedef HRESULT (* STDCALL FindMimeFromData_t) (LPBC pBC,
+typedef HRESULT (WINAPI *FindMimeFromData_t) (LPBC pBC,
 						LPCWSTR pwzUrl,
 						LPVOID pBuffer,
 						DWORD cbSize,
@@ -750,7 +749,7 @@ go_get_file_permissions (char const *uri)
 		file_permissions->others_execute = FALSE;
 #endif /* G_OS_WIN32 */
 	}
-		
+
 	if (info)
 		g_object_unref (info);
 	g_object_unref (file);
@@ -865,7 +864,7 @@ go_file_get_date (char const *uri, GOFileDateType type)
 		tm = (time_t) g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_TIME_CHANGED);
 		break;
 	}
-		
+
 	if (info)
 		g_object_unref (info);
 	g_object_unref (file);
@@ -973,6 +972,7 @@ go_url_encode (gchar const *text, int type)
 	return g_string_free (result, FALSE);
 }
 
+/* You probably want go_gtk_url_show instead! */
 GError *
 go_url_show (gchar const *url)
 {
@@ -1202,17 +1202,12 @@ go_file_access (char const *uri, gint mode)
 		return -1;
 
 #ifdef G_OS_WIN32
-#  ifndef HAVE_G_ACCESS
-#    error "A glib with g_access is required for Win32"
-#  else
 	/* FIXME FIXME FIXME Use Security API instead of checking file attributes only on NT-based environment */
-	ret = g_access (filename, mode);
-#  endif
-#else
-	ret = access (filename, mode);
 #endif
-	
+
+	ret = g_access (filename, mode);
+
 	g_free (filename);
-	
+
 	return ret;
 }
