@@ -35,6 +35,7 @@
 #include <glib/gi18n-lib.h>
 #include <gsf/gsf-impl-utils.h>
 #include <math.h>
+#include <string.h>
 
 static GObjectClass *ppe_parent_klass;
 
@@ -51,8 +52,8 @@ gog_pie_series_element_set_property (GObject *obj, guint param_id,
 	GogPieSeriesElement *pse = GOG_PIE_SERIES_ELEMENT (obj);
 
 	switch (param_id) {
-	case ELEMENT_SEPARATION :
-		pse->separation = g_value_get_float (value);
+	case ELEMENT_SEPARATION:
+		pse->separation = g_value_get_double (value);
 		break;
 
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
@@ -69,8 +70,8 @@ gog_pie_series_element_get_property (GObject *obj, guint param_id,
 	GogPieSeriesElement *pse = GOG_PIE_SERIES_ELEMENT (obj);
 
 	switch (param_id) {
-	case ELEMENT_SEPARATION :
-		g_value_set_float (value, pse->separation);
+	case ELEMENT_SEPARATION:
+		g_value_set_double (value, pse->separation);
 		break;
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 break;
@@ -105,7 +106,7 @@ gog_pie_series_element_class_init (GogPieSeriesElementClass *klass)
 #endif
 
 	g_object_class_install_property (gobject_klass, ELEMENT_SEPARATION,
-		g_param_spec_float ("separation",
+		g_param_spec_double ("separation",
 			_("Separation"),
 			_("Amount a slice is extended as a percentage of the radius"),
 			0, 1000, 0.,
@@ -118,6 +119,54 @@ GSF_DYNAMIC_CLASS (GogPieSeriesElement, gog_pie_series_element,
 
 /*****************************************************************************/
 
+typedef enum {
+	GOG_SHOW_NEGS_SKIP,
+	GOG_SHOW_NEGS_ABSOLUTE,
+	GOG_SHOW_NEGS_WHITE,
+	GOG_SHOW_NEGS_INVERTED, /* not used for now */
+	GOG_SHOW_NEGS_MAX
+} GogShowNegsMode;
+
+static struct {
+	GogShowNegsMode mode;
+	char const *name;
+} neg_modes [GOG_SHOW_NEGS_MAX] = {
+	{GOG_SHOW_NEGS_SKIP, "skip"},
+	{GOG_SHOW_NEGS_ABSOLUTE, "absolute"},
+	{GOG_SHOW_NEGS_WHITE, "white"},
+	{GOG_SHOW_NEGS_INVERTED, "inverted"}
+};
+
+static GogShowNegsMode
+gog_show_neg_mode_from_str (char const *name)
+{
+	unsigned i;
+	GogShowNegsMode ret = GOG_SHOW_NEGS_ABSOLUTE;
+
+	for (i = 0; i < GO_LINE_MAX; i++) {
+		if (strcmp (neg_modes[i].name, name) == 0) {
+			ret = neg_modes[i].mode;
+			break;
+		}
+	}
+	return ret;
+}
+
+static char const *
+gog_show_neg_mode_as_str (GogShowNegsMode mode)
+{
+	unsigned i;
+	char const *ret = "absolute";
+
+	for (i = 0; i < GOG_SHOW_NEGS_MAX; i++) {
+		if (neg_modes[i].mode == mode) {
+			ret = neg_modes[i].name;
+			break;
+		}
+	}
+	return ret;
+}
+
 typedef struct {
 	GogPlotClass	base;
 } GogPiePlotClass;
@@ -127,7 +176,8 @@ enum {
 	PLOT_PROP_INITIAL_ANGLE,
 	PLOT_PROP_DEFAULT_SEPARATION,
 	PLOT_PROP_IN_3D,
-	PLOT_PROP_SPAN
+	PLOT_PROP_SPAN,
+	PLOT_PROP_SHOW_NEGS
 };
 
 GOFFICE_PLUGIN_MODULE_HEADER;
@@ -140,22 +190,32 @@ gog_pie_plot_set_property (GObject *obj, guint param_id,
 			   GValue const *value, GParamSpec *pspec)
 {
 	GogPiePlot *pie = GOG_PIE_PLOT (obj);
-	float f;
 
 	switch (param_id) {
-	case PLOT_PROP_INITIAL_ANGLE :
-		pie->initial_angle = g_value_get_float (value);
+	case PLOT_PROP_INITIAL_ANGLE:
+		pie->initial_angle = g_value_get_double (value);
 		break;
-	case PLOT_PROP_DEFAULT_SEPARATION :
-		f = g_value_get_float (value);
+	case PLOT_PROP_DEFAULT_SEPARATION: {
+		double f = g_value_get_double (value);
 		pie->default_separation = CLAMP (f, 0., 5.);
 		break;
-	case PLOT_PROP_IN_3D :
+	}
+	case PLOT_PROP_IN_3D:
 		pie->in_3d = g_value_get_boolean (value);
 		break;
-	case PLOT_PROP_SPAN :
-		pie->span = g_value_get_float (value);
+	case PLOT_PROP_SPAN:
+		pie->span = g_value_get_double (value);
 		break;
+	case PLOT_PROP_SHOW_NEGS : {
+		GSList *ptr = GOG_PLOT (obj)->series;
+		pie->show_negatives = gog_show_neg_mode_from_str (g_value_get_string (value));
+		/* we need to update all the series */
+		while (ptr) {
+			gog_object_request_update (GOG_OBJECT (ptr->data));
+			ptr = ptr->next;
+		}
+		break;
+	}
 
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 return; /* NOTE : RETURN */
@@ -173,17 +233,20 @@ gog_pie_plot_get_property (GObject *obj, guint param_id,
 	GogPiePlot *pie = GOG_PIE_PLOT (obj);
 
 	switch (param_id) {
-	case PLOT_PROP_INITIAL_ANGLE :
-		g_value_set_float (value, pie->initial_angle);
+	case PLOT_PROP_INITIAL_ANGLE:
+		g_value_set_double (value, pie->initial_angle);
 		break;
-	case PLOT_PROP_DEFAULT_SEPARATION :
-		g_value_set_float (value, pie->default_separation);
+	case PLOT_PROP_DEFAULT_SEPARATION:
+		g_value_set_double (value, pie->default_separation);
 		break;
-	case PLOT_PROP_IN_3D :
+	case PLOT_PROP_IN_3D:
 		g_value_set_boolean (value, pie->in_3d);
 		break;
-	case PLOT_PROP_SPAN :
-		g_value_set_float (value, pie->span);
+	case PLOT_PROP_SPAN:
+		g_value_set_double (value, pie->span);
+		break;
+	case PLOT_PROP_SHOW_NEGS:
+		g_value_set_string (value, gog_show_neg_mode_as_str (pie->show_negatives));
 		break;
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 break;
@@ -238,13 +301,13 @@ gog_pie_plot_class_init (GogPlotClass *plot_klass)
 	gog_klass->view_type	= gog_pie_view_get_type ();
 
 	g_object_class_install_property (gobject_klass, PLOT_PROP_INITIAL_ANGLE,
-		g_param_spec_float ("initial-angle",
+		g_param_spec_double ("initial-angle",
 			_("Initial angle"),
 			_("Degrees clockwise from 12 O'Clock."),
 			0, G_MAXFLOAT, 0.,
 			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, PLOT_PROP_DEFAULT_SEPARATION,
-		g_param_spec_float ("default-separation",
+		g_param_spec_double ("default-separation",
 			_("Default separation"),
 			_("Default amount a slice is extended as a percentage of the radius"),
 			0, G_MAXFLOAT, 0.,
@@ -256,11 +319,17 @@ gog_pie_plot_class_init (GogPlotClass *plot_klass)
 			FALSE,
 			GSF_PARAM_STATIC | G_PARAM_READWRITE));
 	g_object_class_install_property (gobject_klass, PLOT_PROP_SPAN,
-		g_param_spec_float ("span",
+		g_param_spec_double ("span",
 			_("Span"),
 			_("Total angle used as a percentage of the full circle"),
 			10., 100., 100.,
 			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
+	g_object_class_install_property (gobject_klass, PLOT_PROP_SHOW_NEGS,
+		g_param_spec_string ("show-negs",
+			_("Show negative values"),
+			_("How negative values are displayed"),
+			"absolute",
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 
 	{
 		static GogSeriesDimDesc dimensions[] = {
@@ -282,6 +351,7 @@ gog_pie_plot_init (GogPiePlot *pie)
 {
 	pie->base.vary_style_by_element = TRUE;
 	pie->span = 100.;
+	pie->show_negatives = GOG_SHOW_NEGS_ABSOLUTE;
 }
 
 GSF_DYNAMIC_CLASS (GogPiePlot, gog_pie_plot,
@@ -314,8 +384,8 @@ gog_ring_plot_set_property (GObject *obj, guint param_id,
 	GogRingPlot *ring = GOG_RING_PLOT (obj);
 
 	switch (param_id) {
-	case RING_PLOT_PROP_CENTER_SIZE :
-		ring->center_size = g_value_get_float (value);
+	case RING_PLOT_PROP_CENTER_SIZE:
+		ring->center_size = g_value_get_double (value);
 		break;
 
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
@@ -334,8 +404,8 @@ gog_ring_plot_get_property (GObject *obj, guint param_id,
 	GogRingPlot *ring = GOG_RING_PLOT (obj);
 
 	switch (param_id) {
-	case RING_PLOT_PROP_CENTER_SIZE :
-		g_value_set_float (value, ring->center_size);
+	case RING_PLOT_PROP_CENTER_SIZE:
+		g_value_set_double (value, ring->center_size);
 		break;
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 break;
@@ -381,7 +451,7 @@ gog_ring_plot_class_init (GogPiePlotClass *pie_plot_klass)
 #endif
 
 	g_object_class_install_property (gobject_klass, RING_PLOT_PROP_CENTER_SIZE,
-		g_param_spec_float ("center-size",
+		g_param_spec_double ("center-size",
 			_("Center-size"),
 			_("Size of the center hole as a percentage of the radius"),
 			0, 100., 0.,
@@ -435,7 +505,9 @@ find_element (GogView *view, double cx, double cy, double x, double y,
 	vals = go_data_get_values ((*series)->base.values[1].data);
 	scale = 1 / (*series)->total;
 	for (*index = 0 ; *index < (*series)->base.num_elements; (*index)++) {
-		len = fabs (vals[*index]) * scale;
+		len = vals[*index] * scale;
+		if (len < 0.)
+			len = pie->show_negatives? -len: 0.;
 		if (go_finite (len) && len > 1e-3) {
 			theta -= len;
 			if (theta < 0)
@@ -559,6 +631,244 @@ typedef GogPlotViewClass	GogPieViewClass;
 
 static GogViewClass *pie_view_parent_klass;
 
+static int
+gog_pie_view_get_data_at_point (GogPlotView *view, double x, double y, GogSeries **series)
+{
+	GogPiePlot const *model = GOG_PIE_PLOT (view->base.model);
+	GogPieSeries const *pseries = NULL;
+	double r_tot, r_cur, r_int, r_ext, cx, cy, r, th, th0, theta, scale, *vals;
+	double separated_cx, separated_cy;
+	unsigned int index, elem, k;
+	double center_radius;
+	double center_size = 0.0;
+	unsigned num_series = 0;
+	double default_sep, len;
+	double separation_max = 0., separation;
+	GSList *ptr;
+	double outline_width_max = 0.;
+	gboolean has_hole;
+	GogPieSeriesElement *gpse;
+	GList const *overrides;
+	GogShowNegsMode mode = model->show_negatives;
+	gboolean negative;
+	GOStyle *style;
+
+	*series = NULL;
+	/* compute number of valid series */
+	for (ptr = model->base.series ; ptr != NULL ; ptr = ptr->next) {
+		if (!gog_series_is_valid (GOG_SERIES (ptr->data)))
+			continue;
+		pseries = ptr->data;
+		if ((style = go_styled_object_get_style (GO_STYLED_OBJECT (pseries))))
+			outline_width_max = MAX (outline_width_max, gog_renderer_line_size (view->base.renderer, style->line.width));
+		for (overrides = gog_series_get_overrides (GOG_SERIES (pseries));
+		     overrides != NULL;
+		     overrides = overrides->next) {
+			separation = GOG_PIE_SERIES_ELEMENT (overrides->data)->separation;
+			if (separation_max < separation)
+				separation_max = separation;
+			style = go_styled_object_get_style (GO_STYLED_OBJECT (overrides->data));
+			if (outline_width_max < style->line.width)
+				outline_width_max = style->line.width;
+		}
+		num_series++;
+	}
+	if (separation_max < -model->default_separation)
+		separation_max = -model->default_separation;
+
+	if (num_series <= 0)
+		return -1;
+
+	if (GOG_IS_RING_PLOT (model))
+		center_size = GOG_RING_PLOT(model)->center_size;
+	else if (num_series > 1)
+		num_series = 1;
+	/* calculate things when span < 100. */
+	if (model->span < 100.) {
+		double xmin, xmax, ymin, ymax, ratio;
+		double begin, cur, end;
+		if (num_series == 1) {
+			pseries = model->base.series->data;
+			begin = (model->initial_angle + pseries->initial_angle) * M_PI / 180. - M_PI / 2.;
+			end = begin + model->span * M_PI / 50.;
+		} else {
+			/* WARNING: this code has not been checked */
+			ptr = model->base.series;
+			while (!gog_series_is_valid (GOG_SERIES (ptr->data)))
+				ptr = ptr->next;
+			pseries = ptr->data;
+			begin = end = pseries->initial_angle;
+			for (ptr = ptr->next; ptr != NULL; ptr = ptr->next) {
+				if (!gog_series_is_valid (GOG_SERIES (ptr->data)))
+					continue;
+				pseries = ptr->data;
+				cur = pseries->initial_angle;
+				if (cur < begin)
+					begin = cur;
+				if (cur > end)
+					end = cur;
+			}
+			begin = (model->initial_angle + begin) * M_PI / 180. - M_PI / 2.;
+			end = ((model->initial_angle + end) / 180. + model->span / 50.) * M_PI - M_PI / 2.;
+		}
+		cur = ceil (begin / M_PI * 2 - 1e-10) * M_PI / 2;
+		xmin = xmax = cos (begin);
+		ymin = ymax = sin (begin);
+		while (cur < end) {
+			cx = cos (cur);
+			cy = sin (cur);
+			if (cx > xmax)
+				xmax = cx;
+			if (cx < xmin)
+				xmin = cx;
+			if (cy > ymax)
+				ymax = cy;
+			if (cy < ymin)
+				ymin = cy;
+			cur += M_PI / 2;
+		}
+		cx = cos (end);
+		cy = sin (end);
+		if (cx > xmax)
+			xmax = cx;
+		if (cx < xmin)
+			xmin = cx;
+		if (cy > ymax)
+			ymax = cy;
+		if (cy < ymin)
+			ymin = cy;
+		/* we ensure that the center will be visible */
+		if (xmin > 0.)
+			xmin = 0.;
+		if (xmax < 0.)
+			xmax = 0.;
+		if (ymin > 0.)
+			ymin = 0.;
+		if (ymax < 0.)
+			ymax = 0.;
+		ratio = (ymax - ymin) / (xmax - xmin);
+		if (view->base.allocation.h > view->base.allocation.w * ratio) {
+			r_tot = view->base.allocation.w * MAX (xmax, -xmin) / (xmax - xmin);
+			cx = view->base.allocation.x - view->base.allocation.w * xmin / (xmax - xmin);
+			cy = view->base.allocation.y +
+				(view->base.allocation.h + view->base.allocation.w * ratio) / 2.
+				- view->base.allocation.w * ratio * ymax / (ymax - ymin);
+		} else {
+			r_tot = view->base.allocation.h * MAX (ymax, -ymin) / (ymax - ymin);
+			cx = view->base.allocation.x +
+				(view->base.allocation.w - view->base.allocation.h / ratio) / 2.
+				- view->base.allocation.h / ratio * xmin / (xmax - xmin);
+			cy = view->base.allocation.y - view->base.allocation.h * ymin / (ymax - ymin);
+		}
+		r_tot /= 1. + model->default_separation + separation_max;
+	} else {
+		/* centre things */
+		cx = view->base.allocation.x + view->base.allocation.w/2.;
+		cy = view->base.allocation.y + view->base.allocation.h/2.;
+
+		r_tot = view->base.allocation.h;
+		if (r_tot > view->base.allocation.w)
+			r_tot = view->base.allocation.w;
+		r_tot /= 2. * (1. + model->default_separation + separation_max);
+	}
+
+	r = hypot (x - cx, y - cy);
+
+	default_sep = r_tot * model->default_separation;
+	center_radius = r_tot * center_size;
+	r_cur = r_tot * (1. - center_size);
+
+	if (r < center_radius)
+		return -1;
+
+	elem = model->base.index_num;
+	index = 1;
+	th0 = (model->initial_angle + pseries->initial_angle) * M_PI / 180. - M_PI / 2.;
+	th = atan2 (y - cy, x - cx);
+	if (th < th0)
+		th += 2 * M_PI;
+
+	for (ptr = model->base.series ; ptr != NULL ; ptr = ptr->next) {
+		pseries = ptr->data;
+
+		if (!gog_series_is_valid (GOG_SERIES (pseries)))
+			continue;
+		if (index > num_series) /* people snuck extra series into a pie */
+			break;
+
+		if (num_series == index)
+			r_cur -= outline_width_max / 2.0;
+		has_hole = center_radius > 0. || index > 1;
+		r_int = (has_hole)? (center_radius + r_cur * ((double) index - 1.0) / (double) num_series): 0.;
+		r_ext = center_radius + r_cur * (double) index / (double) num_series;
+
+		theta = th0;
+
+		scale = 2 * M_PI / 100 * model->span / pseries->total;
+		vals = go_data_get_values (pseries->base.values[1].data);
+
+		overrides = gog_series_get_overrides (GOG_SERIES (pseries));
+
+		for (k = 0 ; k < pseries->base.num_elements; k++) {
+			len = vals[k] * scale;
+			negative = len < 0;
+			if (negative) {
+				if (mode == GOG_SHOW_NEGS_SKIP) {
+					if ((overrides != NULL) &&
+					    (GOG_SERIES_ELEMENT (overrides->data)->index == k))
+						overrides = overrides->next;
+					continue;
+				}
+				len = -len;
+			}
+			if (!go_finite (len) || len < 1e-3) {
+				if ((overrides != NULL) &&
+				    (GOG_SERIES_ELEMENT (overrides->data)->index == k))
+					overrides = overrides->next;
+				continue;
+			}
+
+			gpse = NULL;
+			if ((overrides != NULL) &&
+			    (GOG_SERIES_ELEMENT (overrides->data)->index == k)) {
+				gpse = GOG_PIE_SERIES_ELEMENT (overrides->data);
+				overrides = overrides->next;
+			    }
+
+			/* only separate the outer ring */
+			separated_cx = cx;
+			separated_cy = cy;
+			if (num_series == index && (default_sep > 0. || gpse != NULL)) {
+
+				separation = default_sep;
+
+				if (gpse != NULL)
+					separation += gpse->separation * r_tot;
+
+				separated_cx += separation * cos (theta + len/2.);
+				separated_cy += separation * sin (theta + len/2.);
+				r = hypot (x - separated_cx, y - separated_cy);
+				th = atan2 (y - separated_cy, x - separated_cx);
+				if (th < th0)
+					th += 2 * M_PI;
+			}
+			theta += len;
+			if (r > r_int && r <= r_ext && th > theta - len && th <= theta) {
+				*series = GOG_SERIES (pseries);
+				return k;
+			}
+			if (gpse) {
+				r = hypot (x - cx, y - cy);
+				th = atan2 (y - cy, x - cx);
+				if (th < th0)
+					th += 2 * M_PI;
+			}
+		}
+		index ++;
+	}
+	return -1;
+}
+
 #define MAX_ARC_SEGMENTS 64
 
 static void
@@ -571,7 +881,7 @@ gog_pie_view_render (GogView *view, GogViewAllocation const *bbox)
 	unsigned elem, k;
 	GOPath *path;
 	GogTheme *theme = gog_object_get_theme (GOG_OBJECT (model));
-	GOStyle *style;
+	GOStyle *style, *white_style = NULL, *elt_style = NULL;
 	GSList *ptr;
 	unsigned num_series = 0;
 	unsigned index;
@@ -583,6 +893,8 @@ gog_pie_view_render (GogView *view, GogViewAllocation const *bbox)
 	double separation_max, separation;
 	GogPieSeriesElement *gpse;
 	GList const *overrides;
+	GogShowNegsMode mode = model->show_negatives;
+	gboolean negative;
 
 	/* compute number of valid series */
 	for (ptr = model->base.series ; ptr != NULL ; ptr = ptr->next) {
@@ -594,11 +906,16 @@ gog_pie_view_render (GogView *view, GogViewAllocation const *bbox)
 
 	if (num_series <= 0)
 		return;
-
+	
 	separation_max = .0;
 	outline_width_max = .0;
 	if ((style = go_styled_object_get_style (GO_STYLED_OBJECT (series))))
 		outline_width_max = gog_renderer_line_size (view->renderer, style->line.width);
+	if (mode == GOG_SHOW_NEGS_WHITE) {
+		white_style = go_style_dup (style);
+		white_style->fill.type = GO_STYLE_FILL_PATTERN;
+		go_pattern_set_solid (&white_style->fill.pattern, GO_COLOR_WHITE);
+	}
 	for (overrides = gog_series_get_overrides (GOG_SERIES (series));
 	     overrides != NULL;
 	     overrides = overrides->next) {
@@ -734,13 +1051,23 @@ gog_pie_view_render (GogView *view, GogViewAllocation const *bbox)
 		vals = go_data_get_values (series->base.values[1].data);
 
 		style = GOG_STYLED_OBJECT (series)->style;
-		if (model->base.vary_style_by_element)
+		if (model->base.vary_style_by_element || mode == GOG_SHOW_NEGS_WHITE)
 			style = go_style_dup (style);
 		gog_renderer_push_style (view->renderer, style);
 
 		overrides = gog_series_get_overrides (GOG_SERIES (series));
 		for (k = 0 ; k < series->base.num_elements; k++) {
-			len = fabs (vals[k]) * scale;
+			len = vals[k] * scale;
+			negative = len < 0;
+			if (negative) {
+				if (mode == GOG_SHOW_NEGS_SKIP) {
+					if ((overrides != NULL) &&
+					    (GOG_SERIES_ELEMENT (overrides->data)->index == k))
+						overrides = overrides->next;
+					continue;
+				}
+				len = -len;
+			}
 			if (!go_finite (len) || len < 1e-3) {
 				if ((overrides != NULL) &&
 				    (GOG_SERIES_ELEMENT (overrides->data)->index == k))
@@ -753,12 +1080,23 @@ gog_pie_view_render (GogView *view, GogViewAllocation const *bbox)
 			    (GOG_SERIES_ELEMENT (overrides->data)->index == k)) {
 				gpse = GOG_PIE_SERIES_ELEMENT (overrides->data);
 				overrides = overrides->next;
-				gog_renderer_push_style (view->renderer,
-					go_styled_object_get_style (
-						GO_STYLED_OBJECT (gpse)));
-			} else if (model->base.vary_style_by_element)
-				gog_theme_fillin_style (theme, style, GOG_OBJECT (series),
-							model->base.index_num + k, GO_STYLE_FILL);
+				if (negative && mode == GOG_SHOW_NEGS_WHITE) {
+					elt_style = go_style_dup (go_styled_object_get_style (
+							GO_STYLED_OBJECT (gpse)));
+					elt_style->fill.type = GO_STYLE_FILL_PATTERN;
+					go_pattern_set_solid (&elt_style->fill.pattern, GO_COLOR_WHITE);
+					gog_renderer_push_style (view->renderer, elt_style);
+				} else
+					gog_renderer_push_style (view->renderer,
+						go_styled_object_get_style (
+							GO_STYLED_OBJECT (gpse)));
+			} else {
+				if (negative && mode == GOG_SHOW_NEGS_WHITE)
+					gog_renderer_push_style (view->renderer, white_style);
+				else if (model->base.vary_style_by_element)
+					gog_theme_fillin_style (theme, style, GOG_OBJECT (series),
+								model->base.index_num + k, GO_STYLE_FILL);
+			}
 
 			/* only separate the outer ring */
 			separated_cx = cx;
@@ -784,13 +1122,20 @@ gog_pie_view_render (GogView *view, GogViewAllocation const *bbox)
 			gog_renderer_draw_shape (view->renderer, path);
 			go_path_clear (path);
 
-			if (gpse != NULL)
+			if (gpse != NULL || (negative && mode == GOG_SHOW_NEGS_WHITE)) {
 				gog_renderer_pop_style (view->renderer);
+				if (elt_style) {
+					g_object_unref (elt_style);
+					elt_style = NULL;
+				}
+			}
 		}
 
 		gog_renderer_pop_style (view->renderer);
 		if (model->base.vary_style_by_element)
 			g_object_unref (style);
+		if (white_style)
+			g_object_unref (white_style);
 
 		index ++;
 	}
@@ -807,13 +1152,37 @@ gog_pie_view_build_toolkit (GogView *view)
 #endif
 }
 
+static char*
+gog_pie_view_get_tip_at_point (GogView *view, double x, double y)
+{
+	GogPieSeries *series = NULL;
+	int index = gog_pie_view_get_data_at_point (GOG_PLOT_VIEW (view), x, y, (GogSeries** )&series);
+	char *label, *ret;
+	double *vals, value;
+
+	if (index < 0)
+		return NULL;
+	vals = go_data_get_values (series->base.values[1].data);
+	value = fabs (vals[index]);
+	label = series->base.values[0].data? go_data_get_vector_string (series->base.values[0].data, index): NULL;
+	if (label && *label)
+		ret = g_strdup_printf (_("%s: %g (%.2f%%)"), label, value, value * 100 / series->total);
+	else
+		ret = g_strdup_printf (_("%g (%.2f%%)"), value, value * 100 / series->total);
+	g_free (label);
+	return ret;
+}
+
 static void
 gog_pie_view_class_init (GogViewClass *view_klass)
 {
+	GogPlotViewClass *pv_klass = (GogPlotViewClass *) view_klass;
 	pie_view_parent_klass = g_type_class_peek_parent (view_klass);
 
 	view_klass->render 		= gog_pie_view_render;
 	view_klass->build_toolkit 	= gog_pie_view_build_toolkit;
+	view_klass->get_tip_at_point    = gog_pie_view_get_tip_at_point;
+	pv_klass->get_data_at_point     = gog_pie_view_get_data_at_point;
 }
 
 GSF_DYNAMIC_CLASS (GogPieView, gog_pie_view,
@@ -837,11 +1206,11 @@ gog_pie_series_set_property (GObject *obj, guint param_id,
 	GogPieSeries *pie = GOG_PIE_SERIES (obj);
 
 	switch (param_id) {
-	case SERIES_PROP_INITIAL_ANGLE :
-		pie->initial_angle = g_value_get_float (value);
+	case SERIES_PROP_INITIAL_ANGLE:
+		pie->initial_angle = g_value_get_double (value);
 		break;
-	case SERIES_PROP_SEPARATION :
-		pie->separation = g_value_get_float (value);
+	case SERIES_PROP_SEPARATION:
+		pie->separation = g_value_get_double (value);
 		break;
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 return; /* NOTE : RETURN */
@@ -858,11 +1227,11 @@ gog_pie_series_get_property (GObject *obj, guint param_id,
 	GogPieSeries *pie = GOG_PIE_SERIES (obj);
 
 	switch (param_id) {
-	case SERIES_PROP_INITIAL_ANGLE :
-		g_value_set_float (value, pie->initial_angle);
+	case SERIES_PROP_INITIAL_ANGLE:
+		g_value_set_double (value, pie->initial_angle);
 		break;
-	case SERIES_PROP_SEPARATION :
-		g_value_set_float (value, pie->separation);
+	case SERIES_PROP_SEPARATION:
+		g_value_set_double (value, pie->separation);
 		break;
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 break;
@@ -876,6 +1245,7 @@ gog_pie_series_update (GogObject *obj)
 	int len = 0;
 	GogPieSeries *series = GOG_PIE_SERIES (obj);
 	unsigned old_num = series->base.num_elements;
+	GogShowNegsMode mode = GOG_PIE_PLOT (series->base.plot)->show_negatives;
 
 	if (series->base.values[1].data != NULL) {
 		vals = go_data_get_values (series->base.values[1].data);
@@ -883,9 +1253,14 @@ gog_pie_series_update (GogObject *obj)
 	}
 	series->base.num_elements = len;
 
-	for (total = 0. ; len-- > 0 ;)
-		if (go_finite (vals[len]))
-			total += fabs (vals[len]);
+	for (total = 0. ; len-- > 0 ;) {
+		double val = vals[len];
+		if (go_finite (val)) {
+			if (val < 0)
+				val = (mode == GOG_SHOW_NEGS_SKIP)? 0.: -val;
+			total += val;
+		}
+	}
 	series->total = total;
 
 	/* queue plot for redraw */
@@ -911,13 +1286,13 @@ gog_pie_series_class_init (GObjectClass *gobject_klass)
 	gobject_klass->get_property = gog_pie_series_get_property;
 
 	g_object_class_install_property (gobject_klass, SERIES_PROP_INITIAL_ANGLE,
-		g_param_spec_float ("initial-angle",
+		g_param_spec_double ("initial-angle",
 			_("Initial-angle"),
 			_("Degrees clockwise from 12 O'Clock"),
 			0, G_MAXFLOAT, 0.,
 			GSF_PARAM_STATIC | G_PARAM_READWRITE));
 	g_object_class_install_property (gobject_klass, SERIES_PROP_SEPARATION,
-		g_param_spec_float ("separation",
+		g_param_spec_double ("separation",
 			_("Separation"),
 			_("Default amount a slice is extended as a percentage of the radius"),
 			0, G_MAXFLOAT, 0.,
